#include "utils.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif

size_t vyGetFileSize(const char *path) {
#ifdef _WIN32
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    WIN32_FILE_ATTRIBUTE_DATA attribs;
    if (!GetFileAttributesExW(wpath, GetFileExInfoStandard, &attribs))
        return 0;
    return (size_t)attribs.nFileSizeHigh << 32 | (size_t)attribs.nFileSizeLow;
#endif
}

void vySetWorkingDirectory(const char *path) {
#ifdef _WIN32
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    SetCurrentDirectoryW(wpath);
#endif
}

uint64_t vyGetCurrentTimestamp(void) {
#ifdef _WIN32
    FILETIME ft;
    GetSystemTimeAsFileTime(&ft);
    uint64_t ts = ft.dwLowDateTime;
    ts |= (uint64_t)ft.dwHighDateTime << 32;
    return ts;
#endif
}

uint64_t vyGetFileModificationTimestamp(vy_file_id fid) {
#ifdef _WIN32
    const char *path = vyGetFilePath(fid);
    if (!path) {
        vyLog("ASSETC", "Tried to get modification timestamp of unknown file.");
        return 0;
    }
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    WIN32_FILE_ATTRIBUTE_DATA attribs;
    if (!GetFileAttributesExW(wpath, GetFileExInfoStandard, &attribs))
        return 0;
    uint64_t ts = attribs.ftLastWriteTime.dwLowDateTime;
    ts |= (uint64_t)attribs.ftLastWriteTime.dwHighDateTime << 32;
    return ts;
#endif
}